@@ -15,7 +15,7 @@ import numpy as nm
 
 from sfepy.base.base import sfepy_config_dir, ordered_iteritems
 from sfepy.base.base import output, get_default, set_defaults, Output, Struct
-from sfepy.base.log_plotter import LogPlotter
+from sfepy.base.log_plotter import draw_data, LogPlotter
 
 _msg_no_live = 'warning: log plot is disabled, install matplotlib and' \
     ' multiprocessing'
@@ -50,8 +50,17 @@ def get_logging_conf(conf, log_name='log'):
 
     return log
 
-def name_to_key(name, ii):
-    return name + (':%d' % ii)
+def iter_names(data_names, igs=None):
+    if igs is None:
+        igs = nm.arange(len(data_names))
+
+    ii = iseq = 0
+    for ig, names in ordered_iteritems(data_names):
+        for ip, name in enumerate(names):
+            if ig in igs:
+                yield ig, ip, ii, iseq, name
+                iseq += 1
+            ii += 1
 
 def read_log(filename):
     """
@@ -69,12 +78,13 @@ def read_log(filename):
     info : dict
         The log plot configuration with subplot numbers as keys.
     """
+    from sfepy.base.base import as_float_or_complex as afc
+
     log = {}
     info = {}
 
     fd = open(filename, 'r')
 
-    last_xval = None
     for line in fd:
         if line[0] == '#':
             ls = line.split(':')
@@ -102,15 +112,16 @@ def read_log(filename):
 
         ls = line.split(':')
 
-        key = ls[0]
+        key = int(ls[0])
         xs, ys, vlines = log.setdefault(key, ([], [], []))
 
-        if (len(ls) == 2) and (last_xval is not None):
-            vlines.append(last_xval)
+        if (len(ls) == 2) and len(log[key][0]):
+            vlines.append(log[key][0][-1])
 
         else:
             try:
-                xval, yval = float(ls[1]), float(ls[2])
+                xval = afc(ls[1])
+                yval = afc(ls[2])
 
             except ValueError:
                 continue
@@ -118,8 +129,6 @@ def read_log(filename):
             xs.append(xval)
             ys.append(yval)
 
-            last_xval = xval
-
     fd.close()
 
     for key, (xs, ys, vlines) in six.iteritems(log):
@@ -127,7 +136,43 @@ def read_log(filename):
 
     return log, info
 
-def plot_log(axs, log, info, xticks=None, yticks=None, groups=None):
+def write_log(output, log, info):
+    xlabels, ylabels, yscales, names, plot_kwargs = zip(*info.values())
+    _write_header(output, xlabels, ylabels, yscales, names, plot_kwargs)
+
+    offset = 0
+    for ig, (xlabel, ylabel, yscale, names, plot_kwargs) \
+        in ordered_iteritems(info):
+        for ip, name in enumerate(names):
+            xs, ys, vlines = log[ip + offset]
+
+            for ir, x in enumerate(xs):
+                output('{}: {}: {:.16e}'.format(ip + offset, x, ys[ir]))
+
+                if x in vlines:
+                    output('%d: -----' % (ip + offset))
+
+        offset += len(names)
+
+    output('# ended: %s' % time.asctime())
+
+def _write_header(output, xlabels, ylabels, yscales, data_names, plot_kwargs):
+    _fmt = lambda x: '%s' % x if x is not None else ''
+
+    output('# started: %s' % time.asctime())
+    output('# groups: %d' % len(data_names))
+    for ig, names in enumerate(data_names):
+        output('#   %d' % ig)
+        output('#     xlabel: "%s", ylabel: "%s", yscales: "%s"'
+                    % (_fmt(xlabels[ig]), _fmt(ylabels[ig]),
+                       yscales[ig]))
+        output('#     names: "%s"' % ', '.join(names))
+        output('#     plot_kwargs: "%s"'
+                    % ', '.join('%s' % ii
+                                for ii in plot_kwargs[ig]))
+
+def plot_log(axs, log, info, xticks=None, yticks=None, xnbins=None, ynbins=None,
+             groups=None, show_legends=True, swap_axes=False):
     """
     Plot log data returned by :func:`read_log()` into a specified figure.
 
@@ -143,8 +188,16 @@ def plot_log(axs, log, info, xticks=None, yticks=None, groups=None):
         The list of x-axis ticks (array or None) for each subplot.
     yticks : list of arrays, optional
         The list of y-axis ticks (array or None) for each subplot.
+    xnbins : list, optional
+        The list of x-axis number of bins (int or None) for each subplot.
+    ynbins : list, optional
+        The list of y-axis number of bins (int or None) for each subplot.
     groups : list, optional
         The list of data groups subplots. If not given, all groups are plotted.
+    show_legends : bool
+        If True, show legends in plots.
+    swap_axes : bool
+        If True, swap the axes of the plots.
     """
     import matplotlib.pyplot as plt
 
@@ -175,17 +228,62 @@ def plot_log(axs, log, info, xticks=None, yticks=None, groups=None):
     if yticks is None:
         yticks = [None] * n_gr
 
-    isub = 0
-    for ii, (xlabel, ylabel, yscale, names, plot_kwargs) in six.iteritems(info):
-        if ii not in groups: continue
+    if xnbins is None:
+        xnbins = [None] * n_gr
+
+    if ynbins is None:
+        ynbins = [None] * n_gr
+
+    isub = offset = 0
+    for ig, (xlabel, ylabel, yscale, names, plot_kwargs) \
+        in ordered_iteritems(info):
+        if ig not in groups:
+            offset += len(names)
+            continue
 
         if axs is None:
             ax = fig.add_subplot(n_row, n_col, isub + 1)
 
         else:
-            ax = axs[ii]
+            ax = axs[ig]
 
-        ax.set_yscale(yscale)
+        if not swap_axes:
+            xnb, ynb = xnbins[isub], ynbins[isub]
+            xti, yti = xticks[isub], yticks[isub]
+            ax.set_yscale(yscale)
+            for ip, name in enumerate(names):
+                xs, ys, vlines = log[ip + offset]
+                draw_data(ax, xs, ys, name, plot_kwargs[ip])
+
+                for x in vlines:
+                    ax.axvline(x, color='k', alpha=0.3)
+
+        else:
+            xlabel, ylabel = ylabel, xlabel
+            xti, yti = yticks[isub], xticks[isub]
+            xnb, ynb = ynbins[isub], xnbins[isub]
+            ax.set_xscale(yscale)
+
+            for ip, name in enumerate(names):
+                xs, ys, vlines = log[ip + offset]
+                draw_data(ax, xs, ys, name, plot_kwargs[ip], swap_axes=True)
+
+                for x in vlines:
+                    ax.axhline(x, color='k', alpha=0.3)
+
+        offset += len(names)
+
+        if xti is not None:
+            ax.set_xticks(xti)
+
+        if yti is not None:
+            ax.set_yticks(yti)
+
+        if xnb is not None:
+            ax.locator_params(tight=True, axis='x', nbins=xnb)
+
+        if ynb is not None:
+            ax.locator_params(tight=True, axis='y', nbins=ynb)
 
         if xlabel:
             ax.set_xlabel(xlabel)
@@ -193,23 +291,9 @@ def plot_log(axs, log, info, xticks=None, yticks=None, groups=None):
         if ylabel:
             ax.set_ylabel(ylabel)
 
-        for ip, name in enumerate(names):
-            xs, ys, vlines = log[name]
-            ax.plot(xs, ys, label=name, **plot_kwargs[ip])
+        if show_legends:
+            ax.legend(loc='best')
 
-            for x in vlines:
-                ax.axvline(x, color='k', alpha=0.3)
-
-        if xticks[isub] is not None:
-            ax.set_xticks(xticks[isub])
-
-        else:
-            ax.locator_params(axis='x', nbins=10)
-
-        if yticks[isub] is not None:
-            ax.set_yticks(yticks[isub])
-
-        ax.legend(loc='best')
         isub += 1
 
     plt.tight_layout(pad=0.5)
@@ -237,7 +321,7 @@ class Log(Struct):
 
     def __init__(self, data_names=None, plot_kwargs=None,
                  xlabels=None, ylabels=None, yscales=None,
-                 is_plot=True, aggregate=100, sleep=1.0,
+                 show_legends=True, is_plot=True, aggregate=100, sleep=1.0,
                  log_filename=None, formats=None):
         """
         Parameters
@@ -256,6 +340,8 @@ class Log(Struct):
             The y axis labels of subplots.
         yscales : list of 'linear' or 'log'
             The y axis scales of subplots.
+        show_legends : bool
+            If True, show legends in plots.
         is_plot : bool
             If True, try to use LogPlotter for plotting.
         aggregate : int
@@ -274,11 +360,13 @@ class Log(Struct):
             mpl = None
 
         Struct.__init__(self,
-                        is_plot=is_plot, aggregate=aggregate, sleep=sleep,
+                        show_legends=show_legends, is_plot=is_plot,
+                        aggregate=aggregate, sleep=sleep,
                         data_names={}, n_arg=0, n_gr=0,
                         data={}, x_values={}, n_calls=0, plot_kwargs={},
                         yscales={}, xlabels={}, ylabels={},
-                        plot_pipe=None, formats={}, output=None)
+                        plot_pipe=None, formats={}, _format_styles={},
+                        output=None)
 
         if data_names is not None:
             n_gr = len(data_names)
@@ -303,16 +391,8 @@ class Log(Struct):
 
         if log_filename is not None:
             self.output = Output('', filename=log_filename)
-            self.output('# started: %s' % time.asctime())
-            self.output('# groups: %d' % n_gr)
-            for ig, names in enumerate(data_names):
-                self.output('#   %d' % ig)
-                self.output('#     xlabel: "%s", ylabel: "%s", yscales: "%s"'
-                            % (xlabels[ig], ylabels[ig], yscales[ig]))
-                self.output('#     names: "%s"' % ', '.join(names))
-                self.output('#     plot_kwargs: "%s"'
-                            % ', '.join('%s' % ii
-                                        for ii in self.plot_kwargs[ig]))
+            _write_header(self.output, xlabels, ylabels, yscales, data_names,
+                          self.plot_kwargs)
 
         if self.is_plot and (not self.can_plot):
             output(_msg_no_live)
@@ -323,6 +403,9 @@ class Log(Struct):
         Add a new data group. Notify the plotting process if it is
         already running.
         """
+        if plot_kwargs is None:
+            plot_kwargs = {}
+
         ig = self.n_gr
         self.n_gr += 1
 
@@ -341,32 +424,22 @@ class Log(Struct):
 
         ii = self.n_arg
         for iseq, name in enumerate(names):
-            key = name_to_key(name, ii)
-            self.data[key] = []
-            ii += 1
+            self.data[ii] = []
 
             if formats is not None:
-                self.formats[key] = formats[iseq]
+                self.formats[ii] = formats[iseq]
             else:
-                self.formats[key] = '%.3e'
+                self.formats[ii] = '{:.3e}'
+
+            self._format_styles[ii] = 0 if '%' in self.formats[ii] else 1
+            ii += 1
 
         self.n_arg = ii
 
         if self.plot_pipe is not None:
             send = self.plot_pipe.send
-            send(['add_axis', ig, names, yscale, xlabel, ylabel])
-
-    def iter_names(self, igs=None):
-        if igs is None:
-            igs = nm.arange(self.n_gr)
-
-        ii = iseq = 0
-        for ig, names in ordered_iteritems(self.data_names):
-            for name in names:
-                if ig in igs:
-                    yield ig, ii, iseq, name
-                    iseq += 1
-                ii += 1
+            send(['add_axis', ig, names, yscale, xlabel, ylabel,
+                  self.plot_kwargs[ig]])
 
     def get_log_name(self):
         return os.path.join(sfepy_config_dir,
@@ -403,14 +476,12 @@ class Log(Struct):
             self.terminate()
             return
 
+        if save_figure: return
+
         ls = len(args), self.n_arg
         if full and (ls[0] != ls[1]):
-            if kwargs:
-                return
-            else:
-                msg = 'log called with wrong number of arguments! (%d == %d)' \
-                      % ls
-                raise IndexError(msg)
+            msg = 'log called with wrong number of arguments! (%d == %d)' % ls
+            raise IndexError(msg)
 
         for ig in igs:
             if (x_values is not None) and (x_values[ig] is not None):
@@ -422,7 +493,7 @@ class Log(Struct):
                     ii = 0
                 self.x_values[ig].append(ii)
 
-        for ig, ii, iseq, name in self.iter_names(igs):
+        for ig, ip, ii, iseq, name in iter_names(self.data_names, igs):
             aux = args[iseq]
             if isinstance(aux, nm.ndarray):
                 aux = nm.array(aux, ndmin = 1)
@@ -430,12 +501,16 @@ class Log(Struct):
                     aux = aux[0]
                 else:
                     raise ValueError('can log only scalars (%s)' % aux)
-            key = name_to_key(name, ii)
-            self.data[key].append(aux)
+            self.data[ii].append(aux)
 
             if self.output:
-                self.output(('%%s: %%s: %s' % self.formats[key])
-                            % (name, self.x_values[ig][-1], aux))
+                if self._format_styles[ii]:
+                    self.output(('{}: {}: %s' % self.formats[ii])
+                                .format(ii, self.x_values[ig][-1], aux))
+
+                else:
+                    self.output(('%%s: %%s: %s' % self.formats[ii])
+                                % (ii, self.x_values[ig][-1], aux))
 
         if self.is_plot and self.can_plot:
             if self.n_calls == 0:
@@ -451,7 +526,8 @@ class Log(Struct):
                                                   self.data_names,
                                                   self.yscales,
                                                   self.xlabels,
-                                                  self.ylabels))
+                                                  self.ylabels,
+                                                  self.plot_kwargs))
                 self.plot_process.daemon = True
                 self.plot_process.start()
 
@@ -473,28 +549,20 @@ class Log(Struct):
     def plot_data(self, igs):
         send = self.plot_pipe.send
 
-        ii = 0
-        for ig, names in ordered_iteritems(self.data_names):
-            if ig in igs:
-                send(['ig', ig])
-                send(['clear'])
-                for ip, name in enumerate(names):
-                    key = name_to_key(name, ii)
-                    try:
-                        send(['plot',
-                              nm.array(self.x_values[ig]),
-                              nm.array(self.data[key]),
-                              self.plot_kwargs[ig][ip]])
-                    except:
-                        msg = "send failed! (%s, %s, %s)!" \
-                              % (ii, name, self.data[key])
-                        raise IOError(msg)
-                    ii += 1
+        for ig in igs:
+            send(['clear', ig])
 
-            else:
-                ii += len(names)
+        for ig, ip, ii, iseq, name in iter_names(self.data_names, igs):
+            try:
+                send(['plot', ig, ip, self.x_values[ig][-1],
+                      self.data[ii][-1]])
+            except:
+                msg = "send failed! (%s, %s, %s)!" % (ii, name, self.data[ii])
+                raise IOError(msg)
+
+        if self.show_legends:
+            send(['legends'])
 
-        send(['legends'])
         send(['continue'])
 
     def plot_vlines(self, igs=None, **kwargs):
@@ -511,12 +579,10 @@ class Log(Struct):
             for ig in igs:
                 x = self.x_values[ig]
                 if len(x):
-                    send(['ig', ig])
-                    send(['vline', x[-1], kwargs])
+                    send(['vline', ig, x[-1], kwargs])
 
             send(['continue'])
 
         if self.output:
-            for ig in igs:
-                for name in self.data_names[ig]:
-                    self.output(name + ': -----')
+            for ig, ip, ii, iseq, name in iter_names(self.data_names, igs):
+                self.output('%d: -----' % ii)
