@@ -1,7 +1,8 @@
 """
 Classes of variables for equations/terms.
 """
-import time
+from __future__ import print_function
+from __future__ import absolute_import
 from collections import deque
 
 import numpy as nm
@@ -9,6 +10,7 @@ import numpy as nm
 from sfepy.base.base import (real_types, complex_types, assert_, get_default,
                              output, OneTypeList, Container, Struct, basestr,
                              iter_dict_of_lists)
+from sfepy.base.timing import Timer
 import sfepy.linalg as la
 from sfepy.discrete.functions import Function
 from sfepy.discrete.conditions import get_condition_value
@@ -19,6 +21,8 @@ from sfepy.discrete.common.dof_info import (DofInfo, EquationMap,
 from sfepy.discrete.fem.lcbc_operators import LCBCOperators
 from sfepy.discrete.common.mappings import get_physical_qps
 from sfepy.discrete.evaluate_variable import eval_real, eval_complex
+import six
+from six.moves import range
 
 is_state = 0
 is_virtual = 1
@@ -56,7 +60,7 @@ def create_adof_conns(conn_info, var_indx=None, active_only=True, verbose=True):
             else:
                 eq = nm.arange(var.n_dof, dtype=nm.int32)
                 eq[var.eq_map.eq_ebc] = -1 - (var.eq_map.eq_ebc + offset)
-                eq[var.eq_map.master] = -1 - (var.eq_map.master + offset)
+                eq[var.eq_map.master] = eq[var.eq_map.slave]
 
         adc = create_adof_conn(eq, econn, var.n_components, offset)
 
@@ -79,7 +83,7 @@ def create_adof_conns(conn_info, var_indx=None, active_only=True, verbose=True):
 
     if verbose:
         output('setting up dof connectivities...')
-        tt = time.clock()
+        timer = Timer(start=True)
 
     adof_conns = {}
 
@@ -104,7 +108,7 @@ def create_adof_conns(conn_info, var_indx=None, active_only=True, verbose=True):
             _assign(adof_conns, info, region, var, field, False)
 
     if verbose:
-        output('...done in %.2f s' % (time.clock() - tt))
+        output('...done in %.2f s' % timer.stop())
 
     return adof_conns
 
@@ -127,7 +131,7 @@ def create_adof_conn(eq, conn, dpn, offset):
         n_el, n_ep = conn.shape
         adc = nm.empty((n_el, n_ep * dpn), dtype=conn.dtype)
         ii = 0
-        for idof in xrange(dpn):
+        for idof in range(dpn):
             aux = nm.take(eq, dpn * conn + idof)
             adc[:, ii : ii + n_ep] = aux + nm.asarray(offset * (aux >= 0),
                                                       dtype=nm.int32)
@@ -135,6 +139,20 @@ def create_adof_conn(eq, conn, dpn, offset):
 
     return adc
 
+def expand_basis(basis, dpn):
+    """
+    Expand basis for variables with several components (DOFs per node), in a
+    way compatible with :func:`create_adof_conn()`, according to `dpn`
+    (DOF-per-node count).
+    """
+    n_c, n_bf = basis.shape[-2:]
+    ebasis = nm.zeros(basis.shape[:2] + (dpn, n_bf * dpn), dtype=nm.float64)
+    for ic in range(n_c):
+        for ir in range(dpn):
+            ebasis[..., n_c*ir+ic, ir*n_bf:(ir+1)*n_bf] = basis[..., ic, :]
+
+    return ebasis
+
 class Variables(Container):
     """
     Container holding instances of Variable.
@@ -148,7 +166,7 @@ class Variables(Container):
         Variable.reset()
 
         obj = Variables()
-        for key, val in conf.iteritems():
+        for key, val in six.iteritems(conf):
             var = Variable.from_conf(key, val, fields)
 
             obj[var.name] = var
@@ -321,8 +339,8 @@ class Variables(Container):
                         regs.append(bc.regions[1])
                         var_names.append(vns[1])
 
-            for i0 in xrange(len(regs) - 1):
-                for i1 in xrange(i0 + 1, len(regs)):
+            for i0 in range(len(regs) - 1):
+                for i1 in range(i0 + 1, len(regs)):
                     if ((var_names[i0] == var_names[i1])
                         and not are_disjoint(regs[i0], regs[i1])):
                         raise ValueError('regions %s and %s are not disjoint!'
@@ -450,6 +468,14 @@ class Variables(Container):
 
             var.setup_initial_conditions(ics, self.di, functions)
 
+        for var_name in self.parameter:
+            var = self[var_name]
+            if hasattr(var, 'special') and ('ic' in var.special):
+                setter, sargs, skwargs = var._get_setter('ic', functions)
+
+                var.set_data(setter(*sargs, **skwargs))
+                output('IC data of %s set by %s()' % (var.name, setter.name))
+
     def set_adof_conns(self, adof_conns):
         """
         Set all active DOF connectivities to `self` as well as relevant
@@ -460,7 +486,7 @@ class Variables(Container):
         for var in self:
             var.adof_conns = {}
 
-        for key, val in adof_conns.iteritems():
+        for key, val in six.iteritems(adof_conns):
             if key[0] in self.names:
                 var = self[key[0]]
                 var.adof_conns[key] = val
@@ -493,7 +519,7 @@ class Variables(Container):
         for var in self.iter_state():
             var.apply_ic(vec, self.di.indx[var.name].start, force_values)
 
-    def strip_state_vector(self, vec, follow_epbc=False):
+    def strip_state_vector(self, vec, follow_epbc=False, svec=None):
         """
         Get the reduced DOF vector, with EBC and PBC DOFs removed.
 
@@ -504,14 +530,15 @@ class Variables(Container):
         assembling. For vectors with state (unknown) variables it should be set
         to False, for assembled vectors it should be set to True.
         """
-        svec = nm.empty((self.adi.ptr[-1],), dtype=self.dtype)
+        if svec is None:
+            svec = nm.empty((self.adi.ptr[-1],), dtype=self.dtype)
         for var in self.iter_state():
             aindx = self.adi.indx[var.name]
             svec[aindx] = var.get_reduced(vec, self.di.indx[var.name].start,
                                           follow_epbc)
         return svec
 
-    def make_full_vec(self, svec, force_value=None):
+    def make_full_vec(self, svec, force_value=None, vec=None):
         """
         Make a full DOF vector satisfying E(P)BCs from a reduced DOF
         vector.
@@ -522,6 +549,8 @@ class Variables(Container):
             The reduced DOF vector.
         force_value : float, optional
             Passing a `force_value` overrides the EBC values.
+        vec : array, optional
+            If given, the buffer for storing the result (zeroed).
 
         Returns
         -------
@@ -537,7 +566,8 @@ class Variables(Container):
             else:
                 svec = self.mtx_lcbc * svec
 
-        vec = self.create_state_vector()
+        if vec is None:
+            vec = self.create_state_vector()
         for var in self.iter_state():
             indx = self.di.indx[var.name]
             aindx = self.adi.indx[var.name]
@@ -672,7 +702,7 @@ class Variables(Container):
 
         if isinstance(data, dict):
 
-            for key, val in data.iteritems():
+            for key, val in six.iteritems(data):
                 try:
                     var = self[key]
 
@@ -698,7 +728,7 @@ class Variables(Container):
         else:
             raise ValueError('unknown data class! (%s)' % data.__class__)
 
-    def set_data_from_state(self, var_names, state, var_names_state):
+    def set_from_state(self, var_names, state, var_names_state):
         """
         Set variables with names in `var_names` from state variables with names
         in `var_names_state` using DOF values in the state vector `state`.
@@ -735,10 +765,10 @@ class Variables(Container):
                 var_info[name] = (False, name)
 
         out = {}
-        for key, indx in di.indx.iteritems():
+        for key, indx in six.iteritems(di.indx):
             var = self[key]
 
-            if key not in var_info.keys(): continue
+            if key not in list(var_info.keys()): continue
             is_part, name = var_info[key]
 
             if is_part:
@@ -747,7 +777,7 @@ class Variables(Container):
             else:
                 aux = vec[indx]
 
-            out.update(var.create_output(aux, extend=extend,
+            out.update(var.create_output(aux, key=name, extend=extend,
                                          fill_value=fill_value,
                                          linearization=linearization))
 
@@ -903,7 +933,10 @@ class Variable(Struct):
             self.primary_var_name = get_default(primary_var_name, None, msg)
             if self.primary_var_name == '(set-to-None)':
                 self.primary_var_name = None
-            self.dof_name = self.primary_var_name
+                self.dof_name = self.name
+
+            else:
+                self.dof_name = self.primary_var_name
 
             if special is not None:
                 self.special = special
@@ -923,11 +956,8 @@ class Variable(Struct):
 
         self.n_dof = self.n_nod * self.n_components
 
-        if self.dof_name is None:
-            dof_name = 'aux'
-        else:
-            dof_name = self.dof_name
-        self.dofs = [dof_name + ('.%d' % ii) for ii in range(self.n_components)]
+        self.dofs = [self.dof_name + ('.%d' % ii)
+                     for ii in range(self.n_components)]
 
     def get_primary(self):
         """
@@ -1036,17 +1066,26 @@ class Variable(Struct):
         if self.history is None: return
 
         self.step = ts.step + 1
-        self.data.rotate()
 
         if self.history > 0:
+            # Copy the current step data to the history data, shift history,
+            # initialize if needed. The current step data are left intact.
+            # Note: cannot use self.data.rotate() due to data sharing with
+            # State.
+            for ii in range(self.history, 0, -1):
+                if self.data[ii] is None:
+                    self.data[ii] = nm.empty_like(self.data[0])
+
+                self.data[ii][:] = self.data[ii - 1]
+
             # Advance evaluate cache.
-            for step_cache in self.evaluate_cache.itervalues():
+            for step_cache in six.itervalues(self.evaluate_cache):
                 steps = sorted(step_cache.keys())
                 for step in steps:
                     if step is None:
                         # Special caches with possible custom advance()
                         # function.
-                        for key, val in step_cache[step].iteritems():
+                        for key, val in six.iteritems(step_cache[step]):
                             if hasattr(val, '__advance__'):
                                 val.__advance__(ts, val)
 
@@ -1164,149 +1203,6 @@ class Variable(Struct):
         else:
             return self.initial_condition
 
-class CloseNodesIterator(Struct):
-
-    def __init__(self, field, create_mesh=True, create_graph=True,
-                 strategy=None):
-        self.field = field
-        self.coors = self.field.get_coor()
-
-        if create_mesh or create_graph:
-            self.mesh = self.field.create_mesh()
-
-        if create_graph:
-            self.graph = self.mesh.create_conn_graph()
-            self.perm = self.get_permutation(strategy=strategy)
-            self.strategy = strategy
-
-        else:
-            self.graph = None
-            self.strategy = None
-
-    def __call__(self, strategy=None):
-        if strategy is None or (strategy != self.strategy):
-            self.perm = self.get_permutation(strategy=strategy)
-            self.strategy = strategy
-
-        self.ii = 0
-        return self
-
-    def get_permutation(self, strategy=None):
-        graph = self.graph
-
-        n_nod = self.coors.shape[0]
-        dtype = nm.int32
-
-        if strategy is None:
-            perm = nm.arange(n_nod, dtype=dtype)
-
-        elif strategy == 'rcm':
-            from sfepy.linalg import rcm
-            perm = rcm(graph)
-
-        elif 'greedy' in strategy:
-            ipop, iin = {'00' : (0, 0),
-                         'e0' : (-1, 0),
-                         '0e' : (0, -1),
-                         'ee' : (-1, -1),
-                         '01' : (0, 1),
-                         }[strategy[-2:]]
-
-            perm_i = nm.empty((n_nod,), dtype=dtype)
-            perm_i.fill(-1)
-
-            n_nod = perm_i.shape[0]
-            num = graph.indptr[1:] - graph.indptr[:-1]
-
-            ir = nm.argmin(num)
-            perm_i[ir] = 0
-            active = [ir]
-            ii = 1
-            while ii < n_nod:
-                ir = active.pop(ipop)
-                row = graph.indices[graph.indptr[ir]:graph.indptr[ir+1]]
-
-                ips = []
-                for ip in row:
-                    if perm_i[ip] < 0:
-                        perm_i[ip] = ii
-                        ii += 1
-                        ips.append(ip)
-                if iin >= 0:
-                    active[iin:iin] = ips
-                else:
-                    active.extend(ips)
-
-            perm = nm.empty_like(perm_i)
-            perm[perm_i] = nm.arange(perm_i.shape[0], dtype=perm.dtype)
-
-        return perm
-
-    def test_permutations(self, strategy='rcm'):
-        from sfepy.linalg import permute_in_place, save_sparse_txt
-
-        save_sparse_txt('graph', self.graph, fmt='%d %d %d\n')
-        graph = self.graph.copy()
-
-        perm = self.get_permutation('rcm')
-
-        g_types = ['00', 'e0', '0e', 'ee', '01']
-        g_names = ['greedy_%s' % ii for ii in g_types]
-        g_perms = [self.get_permutation('greedy_%s' % ii) for ii in g_types]
-
-        c1 = self.mesh.coors
-        d1 = la.norm_l2_along_axis(c1[1:] - c1[:-1])
-        d2 = la.norm_l2_along_axis(c1[perm][1:] - c1[perm][:-1])
-        print d1.min(), d1.mean(), d1.max(), d1.std(), d1.var()
-        print d2.min(), d2.mean(), d2.max(), d2.std(), d2.var()
-        ds = []
-        for g_perm in g_perms:
-            d3 = la.norm_l2_along_axis(c1[g_perm][1:] - c1[g_perm][:-1])
-            ds.append(d3)
-            print d3.min(), d3.mean(), d3.max(), d3.std(), d3.var()
-
-        permute_in_place(graph, perm)
-        save_sparse_txt('graph_rcm', graph, fmt='%d %d %d\n')
-
-        for ii, g_name in enumerate(g_names):
-            graph = self.graph.copy()
-            permute_in_place(graph, g_perms[ii])
-            save_sparse_txt('graph_%s' % g_name, graph, fmt='%d %d %d\n')
-
-        from matplotlib import pyplot as plt
-        n_bins = 30
-        plt.figure()
-        plt.subplot(311)
-        _, bins, ps = plt.hist(d1, n_bins, histtype='bar')
-        plt.legend(ps[0:1], ['default'])
-        plt.subplot(312)
-        plt.hist(d2, bins, histtype='bar')
-        plt.legend(ps[0:1], ['RCM'])
-        plt.subplot(313)
-        _, _, ps = plt.hist(nm.array(ds).T, bins, histtype='bar')
-        plt.legend([ii[0] for ii in ps], g_names)
-        plt.savefig('hist_distances_sub.pdf', transparent=True)
-
-        plt.figure()
-        _, _, ps = plt.hist(nm.array([d1, d2] + ds).T, n_bins, histtype='bar')
-        plt.legend([ii[0] for ii in ps], ['default', 'RCM'] + g_names)
-        plt.savefig('hist_distances.pdf', transparent=True)
-        plt.show()
-
-    def __iter__(self):
-        return self
-
-    def next(self):
-        try:
-            ii = self.perm[self.ii]
-            val = self.coors[ii]
-        except IndexError:
-            raise StopIteration
-
-        self.ii += 1
-
-        return ii, val
-
 class FieldVariable(Variable):
     """
     A finite element field variable.
@@ -1315,9 +1211,9 @@ class FieldVariable(Variable):
     """
 
     def __init__(self, name, kind, field, order=None, primary_var_name=None,
-                 special=None, flags=None, **kwargs):
+                 special=None, flags=None, history=None, **kwargs):
         Variable.__init__(self, name, kind, order, primary_var_name,
-                          special, flags, **kwargs)
+                          special, flags, history=history, **kwargs)
 
         self._set_field(field)
 
@@ -1344,6 +1240,33 @@ class FieldVariable(Variable):
 
         self.dim = field.domain.shape.dim
 
+    def _get_setter(self, kind, functions, **kwargs):
+        """
+        Get the setter function of the variable and its arguments depending in
+        the setter kind.
+        """
+        if not (hasattr(self, 'special') and (kind in self.special)):
+            return
+
+        setter_name = self.special[kind]
+        setter = functions[setter_name]
+
+        region = self.field.region
+        nod_list = self.field.get_dofs_in_region(region)
+        nods = nm.unique(nm.hstack(nod_list))
+
+        coors = self.field.get_coor(nods)
+
+        if kind == 'setter':
+            sargs = (kwargs.get('ts'), coors)
+
+        elif kind == 'ic':
+            sargs = (coors, )
+
+        skwargs = {'region' : region}
+
+        return setter, sargs, skwargs
+
     def get_field(self):
         return self.field
 
@@ -1354,7 +1277,7 @@ class FieldVariable(Variable):
 
         See Also
         --------
-        sfepy.discrete.fem.fields.Field.get_mapping()
+        sfepy.discrete.common.fields.Field.get_mapping
         """
         if region is None:
             region = self.field.region
@@ -1364,7 +1287,7 @@ class FieldVariable(Variable):
                                      return_key=return_key)
         return out
 
-    def get_dof_conn(self, dc_type, is_trace=False):
+    def get_dof_conn(self, dc_type, is_trace=False, trace_region=None):
         """
         Get active dof connectivity of a variable.
 
@@ -1383,7 +1306,7 @@ class FieldVariable(Variable):
 
         else:
             aux = self.field.domain.regions[dc_type.region_name]
-            region = aux.get_mirror_region()
+            region = aux.get_mirror_region(trace_region)
             region_name = region.name
 
         key = (var_name, region_name, dc_type.type, is_trace)
@@ -1412,18 +1335,13 @@ class FieldVariable(Variable):
             self.dt = ts.dt
 
         if hasattr(self, 'special') and ('setter' in self.special):
-            setter_name = self.special['setter']
-            setter = functions[setter_name]
+            setter, sargs, skwargs = self._get_setter('setter', functions,
+                                                      ts=ts)
 
-            region = self.field.region
-            nod_list = self.field.get_dofs_in_region(region)
-            nods = nm.unique(nm.hstack(nod_list))
+            self.set_data(setter(*sargs, **skwargs))
+            output('data of %s set by %s()' % (self.name, setter.name))
 
-            coor = self.field.get_coor(nods)
-            self.set_data(setter(ts, coor, region=region))
-            output('data of %s set by %s()' % (self.name, setter_name))
-
-    def set_data_from_qp(self, data_qp, integral, step=0):
+    def set_from_qp(self, data_qp, integral, step=0):
         """
         Set DOFs of variable using values in quadrature points
         corresponding to the given integral.
@@ -1432,11 +1350,34 @@ class FieldVariable(Variable):
 
         # Field nodes values.
         data = self.field.interp_v_vals_to_n_vals(data_vertex)
-        data = data.squeeze()
+        data = data.ravel()
         self.indx = slice(0, len(data))
 
         self.data[step] = data
 
+    def set_from_mesh_vertices(self, data):
+        """
+        Set the variable using values at the mesh vertices.
+        """
+        ndata = self.field.interp_v_vals_to_n_vals(data)
+        self.set_data(ndata)
+
+    def set_from_function(self, fun, step=0):
+        """
+        Set the variable data (the vector of DOF values) using a function of
+        space coordinates.
+
+        Parameters
+        ----------
+        fun : callable
+            The function of coordinates returning DOF values of shape
+            `(n_coor, n_components)`.
+        step : int, optional
+            The time history step, 0 (default) = current.
+        """
+        _, vv = self.field.set_dofs(fun, self.field.region, self.n_components)
+        self.set_data(vv.ravel(), step=step)
+
     def equation_mapping(self, bcs, var_di, ts, functions, problem=None,
                          warn=False):
         """
@@ -1491,10 +1432,7 @@ class FieldVariable(Variable):
             nods, vv = self.field.set_dofs(fun, region, len(dofs), clean_msg)
             eq = expand_nodes_to_equations(nods, dofs, self.dofs)
 
-            self.initial_condition[eq] = vv
-
-    def get_approximation(self):
-        return self.field
+            self.initial_condition[eq] = nm.ravel(vv)
 
     def get_data_shape(self, integral, integration='volume', region_name=None):
         """
@@ -1504,11 +1442,10 @@ class FieldVariable(Variable):
         ----------
         integral : Integral instance
             The integral describing used numerical quadrature.
-        integration : 'volume', 'plate', 'surface', 'surface_extra' or 'point'
+        integration : 'volume', 'surface', 'surface_extra', 'point' or 'custom'
             The term integration type.
         region_name : str
-            The name of surface region, required when `shape_kind` is
-            'surface'.
+            The name of the region of the integral.
 
         Returns
         -------
@@ -1546,15 +1483,15 @@ class FieldVariable(Variable):
         This should be done, for example, prior to every nonlinear
         solver iteration.
         """
-        for step_cache in self.evaluate_cache.itervalues():
-            for key in step_cache.keys():
+        for step_cache in six.itervalues(self.evaluate_cache):
+            for key in list(step_cache.keys()):
                 if key == step: # Given time step to clear.
                     step_cache.pop(key)
 
     def evaluate(self, mode='val',
                  region=None, integral=None, integration=None,
                  step=0, time_derivative=None, is_trace=False,
-                 dt=None, bf=None):
+                 trace_region=None, dt=None, bf=None):
         """
         Evaluate various quantities related to the variable according to
         `mode` in quadrature points defined by `integral`.
@@ -1573,7 +1510,7 @@ class FieldVariable(Variable):
             The integral defining quadrature points in which the
             evaluation occurs. If None, the first order volume integral
             is created. Must not be None for surface integrations.
-        integration : one of 'volume', 'plate', 'surface', 'surface_extra'
+        integration : 'volume', 'surface', 'surface_extra', or 'point'
             The term integration type. If None, it is derived from
             `integral`.
         step : int, default 0
@@ -1597,6 +1534,10 @@ class FieldVariable(Variable):
             `(n_el, n_qp, n_row, n_col)` with the requested data,
             where `n_row`, `n_col` depend on `mode`.
         """
+        if integration == 'custom':
+            msg = 'cannot use FieldVariable.evaluate() with custom integration!'
+            raise ValueError(msg)
+
         step_cache = self.evaluate_cache.setdefault(mode, {})
         cache = step_cache.setdefault(step, {})
 
@@ -1605,9 +1546,9 @@ class FieldVariable(Variable):
             region = field.region
 
         if is_trace:
-            region = region.get_mirror_region()
+            region = region.get_mirror_region(trace_region)
 
-        if region is not field.region:
+        if (region is not field.region) and not region.is_empty:
             assert_(field.region.contains(region))
 
         if integral is None:
@@ -1770,6 +1711,10 @@ class FieldVariable(Variable):
         # EPBC.
         vec[eq_map.master] = vec[eq_map.slave]
 
+        unused_dofs = self.field.get('unused_dofs')
+        if unused_dofs is not None:
+            vec[:] = self.field.restore_substituted(vec)
+
         return vec
 
     def create_output(self, vec=None, key=None, extend=True, fill_value=None,
@@ -1801,7 +1746,7 @@ class FieldVariable(Variable):
         key = get_default(key, self.name)
 
         aux = nm.reshape(vec,
-                         (self.n_dof / self.n_components, self.n_components))
+                         (self.n_dof // self.n_components, self.n_components))
 
         out = self.field.create_output(aux, self.name, dof_names=self.dofs,
                                        key=key, extend=extend,
@@ -1821,7 +1766,7 @@ class FieldVariable(Variable):
 
         integral = Integral('i_tmp', 1)
 
-        vg = field.get_mapping(field.region, integral, 'volume')
+        vg, _ = field.get_mapping(field.region, integral, 'volume')
 
         diameters = domain.get_element_diameters(cells, vg, mode, square=square)
 
@@ -1836,7 +1781,7 @@ class FieldVariable(Variable):
         vec = self()
 
         n_nod, n_dof, dpn = mesh.n_nod, self.n_dof, self.n_components
-        aux = nm.reshape(vec, (n_dof / dpn, dpn))
+        aux = nm.reshape(vec, (n_dof // dpn, dpn))
 
         ext = self.field.extend_dofs(aux, 0.0)
 
@@ -1853,13 +1798,6 @@ class FieldVariable(Variable):
 
         mesh.write(filename, io='auto', out=out)
 
-    def set_from_mesh_vertices(self, data):
-        """
-        Set the variable using values at the mesh vertices.
-        """
-        ndata = self.field.interp_v_vals_to_n_vals(data)
-        self.set_data(ndata)
-
     def has_same_mesh(self, other):
         """
         Returns
@@ -1916,7 +1854,7 @@ class FieldVariable(Variable):
         """
         Evaluate the variable in the given physical coordinates. Convenience
         wrapper around :func:`Field.evaluate_at()
-        <sfepy.discrete.fem.fields_nodal.H1NodalMixin.evaluate_at()>`, see its
+        <sfepy.discrete.common.fields.Field.evaluate_at()>`, see its
         docstring for more details.
         """
         source_vals = self().reshape((self.n_nod, self.n_components))
@@ -1933,9 +1871,7 @@ class FieldVariable(Variable):
 
         return out
 
-    def set_from_other(self, other, strategy='projection',
-                       search_strategy='kdtree', ordering_strategy='rcm',
-                       close_limit=0.1):
+    def set_from_other(self, other, strategy='projection', close_limit=0.1):
         """
         Set the variable using another variable. Undefined values (e.g. outside
         the other mesh) are set to numpy.nan, or extrapolated.
@@ -1943,30 +1879,14 @@ class FieldVariable(Variable):
         Parameters
         ----------
         strategy : 'projection' or 'interpolation'
-            The strategy to set the values: the L^2 orthogonal projection, or
-            a direct interpolation to the nodes (nodal elements only!)
+            The strategy to set the values: the L^2 orthogonal projection (not
+            implemented!), or a direct interpolation to the nodes (nodal
+            elements only!)
 
         Notes
         -----
         If the other variable uses the same field mesh, the coefficients are
         set directly.
-
-        If the other variable uses the same field mesh, only deformed slightly,
-        it is advisable to provide directly the node ids as a hint where to
-        start searching for a containing element; the order of nodes does not
-        matter then.
-
-        Otherwise (large deformation, unrelated meshes, ...) there are
-        basically two ways:
-        a) query each node (its coordinates) using a KDTree of the other nodes
-        - this completely disregards the connectivity information;
-        b) iterate the mesh nodes so that the subsequent ones are close to each
-        other - then also the elements of the other mesh should be close to each
-        other: the previous one can be used as a start for the directional
-        neighbour element crawling to the target point.
-
-        Not sure which way is faster, depends on implementation efficiency and
-        the particular meshes.
         """
         flag_same_mesh = self.has_same_mesh(other)
 
@@ -1985,24 +1905,7 @@ class FieldVariable(Variable):
         else:
             raise ValueError('unknown interpolation strategy! (%s)' % strategy)
 
-        if search_strategy == 'kdtree':
-            tt = time.clock()
-            iter_nodes = CloseNodesIterator(self.field, create_graph=False)
-            output('iterator: %f s' % (time.clock()-tt))
-
-        elif search_strategy == 'crawl':
-            tt = time.clock()
-            iter_nodes = CloseNodesIterator(self.field, strategy='rcm')
-            output('iterator: %f s' % (time.clock()-tt))
-
-            iter_nodes.test_permutations()
-
-        else:
-            raise ValueError('unknown search strategy! (%s)' % search_strategy)
-
-        perm = iter_nodes.get_permutation(iter_nodes.strategy)
-
-        vals = other.evaluate_at(coors[perm], strategy='general',
+        vals = other.evaluate_at(coors, strategy='general',
                                  close_limit=close_limit)
 
         if strategy == 'interpolation':
