@@ -1,18 +1,17 @@
 from __future__ import absolute_import
 import os
-import time
 
 import numpy as nm
-import numpy.linalg as nla
 import scipy as sc
 
 from sfepy.base.base import output, assert_, get_default, debug, Struct
+from sfepy.base.timing import Timer
 from sfepy.discrete.evaluate import eval_equations
 from sfepy.solvers.ts import TimeStepper
-from sfepy.discrete.fem.meshio import HDF5MeshIO
 from sfepy.solvers import Solver, eig
 from sfepy.linalg import MatrixAction
-from .utils import iter_sym, iter_nonsym, create_pis, create_scalar_pis
+from .utils import iter_sym, iter_nonsym, create_pis, create_scalar_pis,\
+    rm_multi
 import six
 from six.moves import range
 
@@ -73,7 +72,7 @@ class MiniAppBase(Struct):
 
         if self.is_linear:
             output('linear problem, trying to presolve...')
-            tt = time.clock()
+            timer = Timer(start=True)
 
             ev = problem.get_evaluator()
 
@@ -87,7 +86,7 @@ class MiniAppBase(Struct):
             problem.set_linear(True)
             problem.try_presolve(mtx_a)
 
-            output('...done in %.2f s' % (time.clock() - tt))
+            output('...done in %.2f s' % timer.stop())
 
         else:
             problem.set_linear(False)
@@ -113,83 +112,93 @@ class CorrSolution(Struct):
         else:
             yield '', self.state
 
+    def iter_time_steps(self):
+        if hasattr(self, 'n_step') and self.n_step > 0:
+            for ii in range(self.n_step):
+                yield self.get_ts_val(ii)
+        else:
+            yield self
+
+    def get_ts_val(self, step):
+        if hasattr(self, 'states'):
+            states = nm.zeros(self.states.shape, dtype=nm.object)
+            for idx in self.components:
+                state = {k: v[step] for k, v in\
+                         six.iteritems(self.states[idx])}
+                states[idx] = state
+
+            out = CorrSolution(name=self.name,
+                               states=states,
+                               components=self.components)
+
+        else:
+            state = {k: v[step] for k, v in six.iteritems(self.state)}
+            out = CorrSolution(name=self.name,
+                               state=state)
+
+        return out
+
+
 class CorrMiniApp(MiniAppBase):
 
     def __init__(self, name, problem, kwargs):
         MiniAppBase.__init__(self, name, problem, kwargs)
         self.output_dir = self.problem.output_dir
         self.set_default('save_name', None)
-        self.set_default('dump_name', self.save_name)
-        self.set_default('dump_variables', [])
-        self.set_default('save_variables', self.dump_variables)
 
         if self.save_name is not None:
             self.save_name = os.path.normpath(os.path.join(self.output_dir,
                                                            self.save_name))
-        if self.dump_name is not None:
-            self.dump_name = os.path.normpath(os.path.join(self.output_dir,
-                                                           self.dump_name))
 
-    def setup_output(self, save_format=None, dump_format=None,
-                      post_process_hook=None, file_per_var=None):
+    def setup_output(self, save_formats=None, post_process_hook=None,
+                     file_per_var=None):
         """Instance attributes have precedence!"""
-        self.set_default('dump_format', dump_format)
-        self.set_default('save_format', save_format)
+        self.set_default('save_formats', save_formats)
         self.set_default('post_process_hook', post_process_hook)
         self.set_default('file_per_var', file_per_var)
 
     def get_save_name_base(self):
         return self.save_name
 
-    def get_dump_name_base(self):
-        return self.get_save_name_base()
-
-    def get_save_name(self):
+    def get_save_name(self, save_format='.h5', stamp=''):
         save_name_base = self.get_save_name_base()
         if save_name_base is not None:
-            return '.'.join((self.get_save_name_base(), self.save_format))
+            return '.'.join((save_name_base + stamp, save_format))
 
-    def get_dump_name(self):
-        dump_name_base = self.get_dump_name_base()
-        if self.dump_format is not None and dump_name_base is not None:
-            return '.'.join((dump_name_base, self.dump_format))
-
-    def get_output(self, corr_sol, is_dump=False, extend=True, variables=None):
+    def get_output(self, corr_sol, is_dump=False, extend=True,
+                   variables=None, var_map=None):
         if variables is None:
             variables = self.problem.get_variables()
         to_output = variables.state_to_output
 
         if is_dump:
-            var_names = self.dump_variables
             extend = False
 
-        else:
-            var_names = self.save_variables
-
         out = {}
         for key, sol in corr_sol.iter_solutions():
-            for var_name in var_names:
-                if key:
-                    skey = var_name + '_' + key
-
+            for var_name in six.iterkeys(sol):
+                if var_name not in variables.ordered_state\
+                    and var_map is not None\
+                    and var_name in var_map:
+                    vname = var_map[var_name]
                 else:
-                    skey = var_name
+                    vname = var_name
 
                 dof_vector = sol[var_name]
 
                 if is_dump:
-                        var = variables[var_name]
-                        shape = (var.n_dof // var.n_components,
-                                 var.n_components)
-                        out[skey] = Struct(name = 'dump', mode = 'nodes',
-                                           data = dof_vector,
-                                           dofs = var.dofs,
-                                           shape = shape,
-                                           var_name = var_name)
+                    skey = var_name + '_' + key if key else var_name
+                    var = variables[vname]
+                    shape = (var.n_dof // var.n_components,
+                             var.n_components)
+                    out[skey] = Struct(name='dump', mode='vertex',
+                                       data=dof_vector,
+                                       shape=shape,
+                                       var_name=vname)
 
                 else:
                     aux = to_output(dof_vector,
-                                    var_info={var_name: (True, var_name)},
+                                    var_info={vname: (True, var_name)},
                                     extend=extend)
                     if self.post_process_hook is not None:
                         aux = self.post_process_hook(aux, self.problem,
@@ -206,22 +215,28 @@ class CorrMiniApp(MiniAppBase):
 
         return out
 
-    def save(self, state, problem, variables=None):
-        save_name = self.get_save_name()
-        if save_name is not None:
-            extend = not self.file_per_var
-            out = self.get_output(state, extend=extend,
-                                  variables=variables)
+    def save(self, state, problem, variables=None, ts=None, var_map=None):
+        if ts is not None:
+            n_digit = int(nm.log10(ts.n_step)) + 1
+            time_stamp = ('_%s' % ('%%0%dd' % n_digit)) % ts.step
+        else:
+            time_stamp = ''
 
-            problem.save_state(save_name, out=out,
-                               file_per_var=self.file_per_var)
+        for save_format in self.save_formats:
+            if self.get_save_name_base() is not None:
+                if save_format in ['h5']:
+                    save_name = self.get_save_name(save_format)
+                    is_dump, file_per_var, extend = True, False, False,
+                else:
+                    save_name = self.get_save_name(save_format, time_stamp)
+                    file_per_var, is_dump = self.file_per_var, False
+                    extend = not file_per_var
 
-        dump_name = self.get_dump_name()
-        if dump_name is not None:
-            problem.save_state(dump_name,
-                               out=self.get_output(state, is_dump=True,
-                                                   variables=variables),
-                               file_per_var=False)
+                out = self.get_output(state, extend=extend, is_dump=is_dump,
+                                      variables=variables, var_map=var_map)
+
+                problem.save_state(save_name, out=out,
+                                   file_per_var=file_per_var, ts=ts)
 
 class ShapeDimDim(CorrMiniApp):
 
@@ -248,6 +263,10 @@ class ShapeDim(CorrMiniApp):
         corr_sol = CorrSolution(name=self.name,
                                 states=pis,
                                 components=clist)
+
+        self.save(corr_sol, problem,
+                  variables=problem.create_variables([self.variables[0]]))
+
         return corr_sol
 
 class OnesDim(CorrMiniApp):
@@ -259,8 +278,8 @@ class OnesDim(CorrMiniApp):
 
         dim = problem.domain.mesh.dim
         nnod = var.n_nod
-        e00 = nm.zeros((nnod, dim), dtype=nm.float64)
-        e1 = nm.ones((nnod,), dtype=nm.float64)
+        e00 = nm.zeros((nnod, dim), dtype=var.dtype)
+        e1 = nm.ones((nnod,), dtype=var.dtype)
 
         ones = nm.zeros((dim,), dtype=nm.object)
         clist = []
@@ -273,6 +292,9 @@ class OnesDim(CorrMiniApp):
         corr_sol = CorrSolution(name=self.name,
                                 states=ones,
                                 components=clist)
+
+        self.save(corr_sol, problem)
+
         return corr_sol
 
 
@@ -280,30 +302,30 @@ class CorrEval(CorrMiniApp):
     def __call__(self, problem=None, data=None):
         problem = get_default(problem, self.problem)
         expr = self.expression
-        for i in range(len(self.requires)):
-            expr = expr.replace(self.requires[i],
-                                "data['%s']" % self.requires[i])
+        for req in map(rm_multi, self.requires):
+            expr = expr.replace(req, "data['%s']" % req)
 
         val = eval(expr)
 
-        var_name = self.variable
-        clist = ['data']
 
         if type(val) is dict:
             corr_sol = CorrSolution(name=self.name,
-                                    state=val,
-                                    components=clist)
-        else:
+                                    state=val)
+        elif type(val) is nm.ndarray:
             if val.dtype == nm.object:
                 corr_sol = CorrSolution(name=self.name,
                                         states=val,
-                                        components=clist)
+                                        components=['data'])
             else:
                 ndof, ndim = val.shape
-                state = {var_name: val.reshape((ndof * ndim,))}
+                state = {self.variable: val.reshape((ndof * ndim,))}
                 corr_sol = CorrSolution(name=self.name,
-                                        state=state,
-                                        components=clist)
+                                        state=state)
+        else:
+            corr_sol = val
+
+        cvars = problem.create_variables([self.variable])
+        self.save(corr_sol, problem, variables=cvars)
 
         return corr_sol
 
@@ -477,7 +499,6 @@ class CorrSetBCS(CorrMiniApp):
         epbcs = Conditions.from_conf(conf_epbc, problem.domain.regions)
 
         conf_variables = select_by_names(problem.conf.variables, self.variable)
-        problem.set_variables(conf_variables)
         variables = Variables.from_conf(conf_variables, problem.fields)
         variables.equation_mapping(ebcs, epbcs, problem.ts, problem.functions)
         state = State(variables)
@@ -487,7 +508,7 @@ class CorrSetBCS(CorrMiniApp):
         corr_sol = CorrSolution(name=self.name,
                                 state=state.get_parts())
 
-        self.save(corr_sol, problem, variables)
+        self.save(corr_sol, problem, variables=variables)
 
         return corr_sol
 
@@ -564,14 +585,14 @@ class PressureEigenvalueProblem(CorrMiniApp):
         output('full A size: %.3f MB' % (8.0 * nm.prod(mtx_a.shape) / 1e6))
         output('full B size: %.3f MB' % (8.0 * nm.prod(mtx_bt.shape) / 1e6))
 
-        ls = Solver.any_from_conf(self.problem.ls_conf,
-                                   presolve=True, mtx=mtx_a)
+        ls = Solver.any_from_conf(self.problem.ls_conf
+                                  + Struct(use_presolve=True), mtx=mtx_a)
         if self.mode == 'explicit':
-            tt = time.clock()
+            timer = Timer(start=True)
             mtx_aibt = nm.zeros(mtx_bt.shape, dtype=mtx_bt.dtype)
             for ic in range(mtx_bt.shape[1]):
                 mtx_aibt[:,ic] = ls(mtx_bt[:,ic].toarray().squeeze())
-            output('mtx_aibt: %.2f s' % (time.clock() - tt))
+            output('mtx_aibt: %.2f s' % timer.stop())
             action_aibt = MatrixAction.from_array(mtx_aibt)
         else:
             ##
@@ -674,7 +695,7 @@ class TCorrectorsViaPressureEVP(CorrMiniApp):
                            lcbc_names=self.get('lcbcs', []))
         problem.update_materials() # Assume parameters constant in time.
 
-    def compute_correctors(self, evp, sign, state0, ts, dump_name, save_name,
+    def compute_correctors(self, evp, sign, state0, ts,
                            problem=None, vec_g=None):
         problem = get_default(problem, self.problem)
 
@@ -689,7 +710,7 @@ class TCorrectorsViaPressureEVP(CorrMiniApp):
                     % (vec_g.max(), vec_g.min()))
             one = nm.ones((nc,), dtype=nm.float64)
 
-        vu, vp = self.dump_variables
+        vu, vp = self.up_variables
 
         variables = problem.get_variables()
         var_u = variables[vu]
@@ -698,10 +719,6 @@ class TCorrectorsViaPressureEVP(CorrMiniApp):
         ##
         # follow_epbc = False -> R1 = - R2 as required. ? for other correctors?
         vec_p0 = sign * var_p.get_reduced(state0[vp], follow_epbc=False)
-##         print state0
-##         print vec_p0
-##         print vec_p0.min(), vec_p0.max(), nla.norm(vec_p0)
-##         debug()
 
         # xi0 = Q^{-1} p(0) = Q^T G p(0)
         vec_xi0 = sc.dot(mtx_q.T, sc.dot(mtx['G'],
@@ -712,6 +729,8 @@ class TCorrectorsViaPressureEVP(CorrMiniApp):
         iee_e_qg = 0.0
         format = '====== time %%e (step %%%dd of %%%dd) ====='\
                  % ((ts.n_digit,) * 2)
+        vu, vp = self.up_variables
+        state = {k: [] for k in [vu, vp, 'd' + vp]}
         for step, time in ts:
             output(format % (time, step + 1, ts.n_step))
 
@@ -726,99 +745,77 @@ class TCorrectorsViaPressureEVP(CorrMiniApp):
             vec_p = sc.dot(mtx_q, e_e_qp + iee_e_qg)
             vec_dp = - sc.dot(mtx_q, (eigs * e_e_qp - e_e_qg))
             vec_u = action_aibt(vec_dp)
-##             bbb = sc.dot(vec_dp.T, - mtx['C'] * vec_p0)
 
             vec_u = var_u.get_full(vec_u)
             vec_p = var_p.get_full(vec_p)
             # BC nodes - time derivative of constant is zero!
             vec_dp = var_p.get_full(vec_dp, force_value=0.0)
-##             aaa = sc.dot(vec_xi0.T, eigs * (eigs * e_e_qp))
-##             print aaa
-##             print bbb
 
-            self.save(dump_name, save_name, vec_u, vec_p, vec_dp, ts, problem)
+            state[vu].append(vec_u)
+            state[vp].append(vec_p)
+            state['d' + vp].append(vec_dp)
 
-    def save(self, dump_name, save_name, vec_u, vec_p, vec_dp, ts, problem):
-        """
-        1. saves raw correctors into hdf5 files (filename)
-        2. saves correctors transformed to output for visualization
-        """
-        vu, vp = self.dump_variables
-        out = {vu : Struct(name='dump', mode='nodes', data=vec_u,
-                           dofs=None, var_name=vu),
-               vp : Struct(name='dump', mode='nodes', data=vec_p,
-                           dofs=None, var_name=vp),
-               'd'+vp : Struct(name='dump', mode='nodes', data=vec_dp,
-                               dofs=None, var_name=vp)}
+        return {k: nm.asarray(v) for k, v in state.items()}
 
-        problem.save_state(dump_name, out=out, file_per_var=False, ts=ts)
+    def save(self, corrs, problem, ts):
+        ts0 = TimeStepper(0, 1)
+        ts0.set_from_ts(ts, step=0)
+        _, vp = self.up_variables
 
-        # For visualization...
-        out = {}
-        extend = not self.file_per_var
+        for step, _ in ts0:
+            icorrs = corrs.get_ts_val(step)
+            super(TCorrectorsViaPressureEVP, self).save(icorrs, problem, ts=ts0,
+                                                        var_map={'d' + vp: vp})
 
-        variables = self.problem.get_variables()
-        to_output = variables.state_to_output
 
-        out.update(to_output(vec_u, var_info={vu : (True, vu)},
-                             extend=extend))
-        out.update(to_output(vec_p, var_info={vp : (True, vp)},
-                             extend=extend))
-        out.update(to_output(vec_dp, var_info={vp : (True, 'd'+vp)},
-                             extend=extend))
-        if self.post_process_hook is not None:
-            out = self.post_process_hook(out, problem,
-                                          {vu : vec_u,
-                                           vp : vec_p, 'd'+vp : vec_dp},
-                                          extend=extend)
-        problem.save_state(save_name, out=out,
-                            file_per_var=self.file_per_var, ts=ts)
+def create_ts_coef(cls):
+    """
+    Define a new class with modified call method which accepts
+    time dependent data (correctors).
+    """
+    class TSCoef(cls):
 
-    def verify_correctors(self, sign, state0, filename, problem=None):
+        def __call__(self, volume=None, problem=None, data=None):
+            problem = get_default(problem, self.problem)
+            ts_keys = []
+            ts_data = {}
+            n_step = None
+            for key, val in six.iteritems(data):
+                if isinstance(val, CorrSolution) and hasattr(val, 'n_step'):
+                    if n_step is None:
+                        n_step = val.n_step
+                    else:
+                        if not(n_step == val.n_step):
+                            raise ValueError('incorrect number of time' +\
+                                             'steps in %s!' % self.name)
 
-        problem = get_default(problem, self.problem)
+                    ts_keys.append(key)
+                else:
+                    ts_data[key] = val
 
-        io = HDF5MeshIO(filename)
-        ts = TimeStepper(*io.read_time_stepper())
+            if n_step is None:
+                raise ValueError('no time steps found in %s!' % self.name)
 
-        ts.set_step(0)
-        problem.equations.init_time(ts)
+            n_digit = int(nm.log10(n_step))+1
+            format = '====== step %%%dd of %%%dd =====' % ((n_digit,) * 2)
+            out = []
+            for step in range(n_step):
+                output(format % (step + 1, n_step))
+                for key in ts_keys:
+                    ts_data[key] = data[key].get_ts_val(step)
+                out.append(cls.__call__(self, volume, problem, ts_data))
 
-        variables = self.problem.get_variables()
+            out = nm.asarray(out)
+            sh = out.shape
+            if len(sh) == 2:
+                out = out.reshape((sh[0], 1, sh[1]))
+            elif len(sh) == 1:
+                out = out.reshape((sh[0], 1, 1))
 
-        vu, vp = self.dump_variables
-        vdp = self.verify_variables[-1]
+            return out
 
-        p0 = sign * state0[vp]
+    return TSCoef
 
-        format = '====== time %%e (step %%%dd of %%%dd) ====='\
-                 % ((ts.n_digit,) * 2)
-        vv = variables
-        ok = True
-        for step, time in ts:
-            output(format % (time, step + 1, ts.n_step))
-
-            data = io.read_data(step)
-            if step == 0:
-                assert_(nm.allclose(data[vp].data, p0))
-
-            state0 = problem.create_state()
-            state0.set_full(data[vu].data, vu)
-            state0.set_full(data[vp].data, vp)
-            vv[vdp].set_data(data['d'+vp].data)
-
-            problem.update_time_stepper(ts)
-            state = problem.solve(state0)
-            state, state0 = state(), state0()
-            err = nla.norm(state - state0) / nla.norm(state0)
-            output(state.min(), state.max())
-            output(state0.min(), state0.max())
-            output('>>>>>', err)
-
-            ok = ok and (err < 1e-12)
-            problem.advance(ts)
-
-        return ok
 
 class CoefDummy(MiniAppBase):
     """
@@ -832,7 +829,8 @@ class TSTimes(MiniAppBase):
     """Coefficient-like class, returns times of the time stepper."""
     def __call__(self, volume=None, problem=None, data=None):
         problem = get_default(problem, self.problem)
-        return problem.get_time_solver().ts.times
+        problem.init_solvers()
+        return problem.get_timestepper().times
 
 class VolumeFractions(MiniAppBase):
     """Coefficient-like class, returns volume fractions of given regions within
@@ -845,8 +843,9 @@ class VolumeFractions(MiniAppBase):
             vkey = 'volume_%s' % region_name
             key = 'fraction_%s' % region_name
 
-            equations, variables = problem.create_evaluable(self.expression % region_name)
-            val = eval_equations(equations, variables)
+            equations, variables = problem.create_evaluable(
+                self.expression % region_name)
+            val = eval_equations(equations, variables).real
 
             vf[vkey] = nm.asarray(val, dtype=nm.float64)
             vf[key] = vf[vkey] / self._get_volume(volume)
@@ -946,49 +945,6 @@ class CoefNonSymNonSym(CoefSymSym):
     is_sym = False
 
 
-class CoefFMSymSym(MiniAppBase):
-    """
-    Fading memory sym x sym coefficients.
-    """
-
-    def __call__(self, volume, problem=None, data=None):
-        problem = get_default(problem, self.problem)
-
-        dim, sym = problem.get_dim(get_sym=True)
-
-        filename = self.set_variables(None, None, None, 0, 0,
-                                      'filename', **data)
-        ts = TimeStepper(*HDF5MeshIO(filename).read_time_stepper())
-
-        coef = nm.zeros((ts.n_step, sym, sym), dtype=self.dtype)
-
-        term_mode = self.term_mode
-        equations, variables = problem.create_evaluable(self.expression,
-                                                        term_mode=term_mode)
-
-        for ir, (irr, icr) in enumerate(iter_sym(dim)):
-            filename = self.set_variables(None, None, None, irr, icr,
-                                          'filename', **data)
-            io = HDF5MeshIO(filename)
-
-            for step, time in ts:
-                self.set_variables(variables, io, step, None, None,
-                                   'row', **data)
-
-                for ic, (irc, icc) in enumerate(iter_sym(dim)):
-                    self.set_variables(variables, None, None, irc, icc,
-                                       'col', **data)
-
-                    val = eval_equations(equations, variables,
-                                         term_mode=term_mode)
-
-                    coef[step,ir,ic] = val
-
-        coef /= self._get_volume(volume)
-
-        return coef
-
-
 class CoefDimSym(CoefNN):
     def __call__(self, volume, problem=None, data=None):
         problem = get_default(problem, self.problem)
@@ -1053,47 +1009,19 @@ class CoefNonSym(CoefSym):
     is_sym = False
 
 
-class CoefFMSym(MiniAppBase):
-    """
-    Fading memory sym coefficients.
-    """
-
-    def __call__(self, volume, problem=None, data=None):
-        problem = get_default(problem, self.problem)
-
-        dim, sym = problem.get_dim(get_sym=True)
-
-        filename = self.set_variables(None, 0, 0, 'filename', **data)
-        ts = TimeStepper(*HDF5MeshIO(filename).read_time_stepper())
-
-        coef = nm.zeros((ts.n_step, sym), dtype=self.dtype)
-
-        term_mode = self.term_mode
-        equations, variables = problem.create_evaluable(self.expression,
-                                                        term_mode=term_mode)
-
-        self.set_variables(variables, None, None, 'col', **data)
-
-        for ii, (ir, ic) in enumerate(iter_sym(dim)):
-            filename = self.set_variables(None, ir, ic, 'filename', **data)
-            io = HDF5MeshIO(filename)
-            for step, time in ts:
-                self.set_variables(variables, io, step, 'row', **data)
-
-                val = eval_equations(equations, variables,
-                                     term_mode=term_mode)
-
-                coef[step,ii] = val
-
-        coef /= self._get_volume(volume)
-
-        return coef
-
 class CoefOne(MiniAppBase):
 
     def set_variables_default(variables, set_var, data):
         for (var, req, comp) in set_var:
-            variables[var].set_data(data[req].state[comp])
+            if type(req) is tuple:
+                val = data[req[0]].state[comp].copy()
+                for ii in req[1:]:
+                    val += data[ii].state[comp]
+
+            else:
+                val = data[req].state[comp]
+
+            variables[var].set_data(val)
 
     set_variables_default = staticmethod(set_variables_default)
 
@@ -1104,11 +1032,12 @@ class CoefOne(MiniAppBase):
         equations, variables = problem.create_evaluable(self.expression,
                                                         term_mode=term_mode)
 
-        if isinstance(self.set_variables, list):
-            self.set_variables_default(variables, self.set_variables,
-                                       data)
-        else:
-            self.set_variables(variables, **data)
+        if hasattr(self, 'set_variables'):
+            if isinstance(self.set_variables, list):
+                self.set_variables_default(variables, self.set_variables,
+                                           data)
+            else:
+                self.set_variables(variables, **data)
 
         val = eval_equations(equations, variables,
                              term_mode=term_mode)
@@ -1117,45 +1046,13 @@ class CoefOne(MiniAppBase):
 
         return coef
 
-class CoefFMOne(MiniAppBase):
-    """
-    Fading memory scalar coefficients.
-    """
-
-    def __call__(self, volume, problem=None, data=None):
-        problem = get_default(problem, self.problem)
-
-        filename = self.set_variables(None, None, None, 'filename', **data)
-        io = HDF5MeshIO(filename)
-        ts = TimeStepper(*io.read_time_stepper())
-
-        coef = nm.zeros((ts.n_step, 1), dtype=self.dtype)
-
-        term_mode = self.term_mode
-        equations, variables = problem.create_evaluable(self.expression,
-                                                        term_mode=term_mode)
-
-        self.set_variables(variables, None, None, 'col', **data)
-
-        for step, time in ts:
-            self.set_variables(variables, io, step, 'row', **data)
-
-            val = eval_equations(equations, variables,
-                                 term_mode=term_mode)
-
-            coef[step] = val
-
-        coef /= self._get_volume(volume)
-
-        return coef
 
 class CoefSum(MiniAppBase):
 
     def __call__(self, volume, problem=None, data=None):
-
         coef = nm.zeros_like(data[self.requires[0]])
-        for i in range(len(self.requires)):
-            coef += data[self.requires[i]]
+        for req in map(rm_multi, self.requires):
+            coef += data[req]
 
         return coef
 
@@ -1164,11 +1061,9 @@ class CoefEval(MiniAppBase):
     Evaluate expression.
     """
     def __call__(self, volume, problem=None, data=None):
-
         expr = self.expression
-        for i in range(len(self.requires)):
-            expr = expr.replace(self.requires[i],
-                                "data['%s']" % self.requires[i])
+        for req in map(rm_multi, self.requires):
+            expr = expr.replace(req, "data['%s']" % req)
 
         coef = eval(expr)
 
