@@ -4,7 +4,7 @@ import numpy as nm
 from .bspline import BSpline
 from sfepy.base.base import Struct
 from six.moves import range
-
+from sfepy.base.compat import lstsq
 
 class SplineBox(Struct):
     """
@@ -62,15 +62,12 @@ class SplineBox(Struct):
             ncp.append(ncp0)
             cp.append(cp0)
             ncp_tot *= ncp0
-            print(knots)
-            print(cp0)
 
         cp_coors = nm.zeros((ncp_tot, cdim), dtype=nm.double)
         cp_idx, mul_cp_idx = SplineBox.gen_cp_idxs(ncp)
         for ii in range(cdim):
             cp_coors[:, ii] = cp[ii][cp_idx[ii]]
 
-        print(cp_coors)
         return {'base': base,
                 'uidx': uidx,
                 'ncp': ncp,
@@ -115,7 +112,7 @@ class SplineBox(Struct):
                 b = field
 
             a = self.get_box_matrix()
-            self.cp_values = nm.linalg.lstsq(a, b)[0]
+            self.cp_values = lstsq(a, b)[0]
 
         else:
             self.cp_values = self.cp_coors
@@ -255,7 +252,7 @@ class SplineBox(Struct):
         return nm.dot(aux[:, nm.newaxis],
                       nm.reshape(dirvec, (1, self.cp_values.shape[1])))
 
-    def write_control_net(self, filename):
+    def write_control_net(self, filename, deform_by_values=True):
         """
         Write the SplineBox shape to the VTK file.
 
@@ -278,7 +275,8 @@ class SplineBox(Struct):
         elif self.cdim == 3:
             ptformat = "%e %e %e\n"
 
-        for cpt in self.cp_coors:
+        cp_coors = self.cp_values if deform_by_values else self.cp_coors
+        for cpt in cp_coors:
             f.write(ptformat % tuple(cpt))
 
         cells = nm.array([nm.arange(0, ncp[0] - 1), nm.arange(1, ncp[0])]).T
@@ -403,7 +401,7 @@ class SplineRegion2D(SplineBox):
         bnd_poly.append(bnd_poly[0][0, :])
         ncpoints = 1
         base, bspl, uidx, ncp = [], [], [], []
-        for idim, si in enumerate([0, 1]):
+        for si in [0, 1]:
             s = spl_bnd[si]
             bspl0 = BSpline(s.degree, ncp=s.ncp)
             bspl0.set_knot_vector(s.knots)
