@@ -1,3 +1,4 @@
+from __future__ import absolute_import
 from copy import copy
 
 import numpy as nm
@@ -7,10 +8,10 @@ from sfepy.base.base import ordered_iteritems
 from sfepy import data_dir
 
 filename_meshes = [data_dir + '/meshes/elements/%s_2.mesh' % geom
-                   for geom in ['1_2', '2_3', '2_4', '3_4', '3_8']]
+                   for geom in ['1_2', '2_3', '2_4', '3_4', '3_8', '3_2_4']]
 
 def make_term_args(arg_shapes, arg_kinds, arg_types, ats_mode, domain,
-                   material_value=None):
+                   material_value=None, poly_space_base=None):
     from sfepy.base.base import basestr
     from sfepy.discrete import FieldVariable, Material, Variables, Materials
     from sfepy.discrete.fem import Field
@@ -26,6 +27,9 @@ def make_term_args(arg_shapes, arg_kinds, arg_types, ats_mode, domain,
             if sh == 'D':
                 return dim
 
+            elif sh == 'D2':
+                return dim**2
+
             elif sh == 'S':
                 return sym
 
@@ -66,7 +70,8 @@ def make_term_args(arg_shapes, arg_kinds, arg_types, ats_mode, domain,
         if arg_kind.endswith('variable'):
             shape = _parse_scalar_shape(sh[0] if isinstance(sh, tuple) else sh)
             field = Field.from_args('f%d' % ii, nm.float64, shape, omega,
-                                    approx_order=1)
+                                    approx_order=1,
+                                    poly_space_base=poly_space_base)
 
             if arg_kind == 'virtual_variable':
                 if sh[1] is not None:
@@ -111,9 +116,16 @@ def make_term_args(arg_shapes, arg_kinds, arg_types, ats_mode, domain,
 
             shape = _parse_tuple_shape(sh)
             if (len(shape) > 1) or (shape[0] > 1):
-                # Array.
-                val = nm.empty(shape, dtype=nm.float64)
-                val.fill(material_value)
+                if ((len(shape) == 2) and (shape[0] ==  shape[1])
+                    and (material_value != 0.0)):
+                    # Identity matrix.
+                    val = nm.eye(shape[0], dtype=nm.float64)
+
+                else:
+                    # Array.
+                    val = nm.empty(shape, dtype=nm.float64)
+                    val.fill(material_value)
+
                 values = {'%sc%d' % (prefix, ii)
                           : val}
 
@@ -162,8 +174,12 @@ class Test(TestCommon):
             domains.append(domain)
 
         integral = Integral('i', order=3)
+        qp_coors, qp_weights = integral.get_qp('3_8')
+        custom_integral = Integral('i', coors=qp_coors, weights=qp_weights,
+                                   order='custom')
 
         test = Test(domains=domains, integral=integral,
+                    custom_integral=custom_integral,
                     conf=conf, options=options)
         return test
 
@@ -175,13 +191,16 @@ class Test(TestCommon):
         for domain in self.domains:
             self.report('domain: %s' % domain.name)
 
+            domain_geometry = list(domain.geom_els.values())[0].name
+            if domain.shape.dim != domain.shape.tdim:
+                domain_geometry = '%d_%s' % (domain.shape.dim, domain_geometry)
+
             for _, term_cls in ordered_iteritems(term_table):
-                if not domain.geom_els.values()[0].name in term_cls.geometries:
+                if not domain_geometry in term_cls.geometries:
                     continue
 
-                rname = 'Omega' \
-                        if term_cls.integration in ('volume', 'point') \
-                        else 'Gamma'
+                vint = ('volume', 'point', 'custom')
+                rname = 'Omega' if term_cls.integration in vint else 'Gamma'
 
                 self.report('<-- %s ...' % term_cls.name)
 
@@ -220,6 +239,14 @@ class Test(TestCommon):
         if not isinstance(arg_shapes_list, list):
             arg_shapes_list = [arg_shapes_list]
 
+        if term_cls.integration != 'custom':
+            integral = self.integral
+
+        else:
+            integral = self.custom_integral
+
+        poly_space_base = getattr(term_cls, 'poly_space_base', 'lagrange')
+
         prev_shapes = {}
         for _arg_shapes in arg_shapes_list:
             # Unset shapes are taken from the previous iteration.
@@ -245,14 +272,14 @@ class Test(TestCommon):
                 else:
                     material_value = 1.0
                 aux = make_term_args(arg_shapes, arg_kinds, ats, mode, domain,
-                                     material_value=material_value)
+                                     material_value=material_value,
+                                     poly_space_base=poly_space_base)
                 args, str_args, materials, variables = aux
 
                 self.report('args:', str_args)
 
                 name = term_call % (', '.join(str_args))
-                term = Term.new(name, self.integral,
-                                domain.regions[rname], **args)
+                term = Term.new(name, integral, domain.regions[rname], **args)
                 term.setup()
 
                 call_mode = 'weak' if term.names.virtual else 'eval'
@@ -266,6 +293,9 @@ class Test(TestCommon):
 
                 else:
                     vals, iels, status = out
+
+                if isinstance(vals, tuple):
+                    # Dynamic connectivity terms.
                     vals = vals[0]
 
                 _ok = nm.isfinite(vals).all()
@@ -277,7 +307,7 @@ class Test(TestCommon):
 
                 _ok = status == 0
                 if not _ok:
-                    self.report('status is %d!' % status)
+                    self.report('status is %s!' % status)
 
                 ok = ok and _ok
 
@@ -289,13 +319,15 @@ class Test(TestCommon):
                         vals, iels, status = term.evaluate(mode=call_mode,
                                                            diff_var=svar.name,
                                                            ret_status=True)
-                        vals = vals[0]
+                        if isinstance(vals, tuple):
+                            # Dynamic connectivity terms.
+                            vals = vals[0]
 
                         _ok = status == 0
                         ok = ok and _ok
                         self.report('diff: %s' % svar.name)
                         if not _ok:
-                            self.report('status is %d!' % status)
+                            self.report('status is %s!' % status)
 
                         _ok = nm.isfinite(vals).all()
                         ok = ok and _ok
